/*
    Tickle class library

    Copyright (c) 2003,2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef EMU_BITBLOCK_H_
#define EMU_BITBLOCK_H_

#include <string.h>

#include "emu_blitter.h"

enum TBitOp {
    opSrcTrans  = 0x0004 | 0,
    opDstTrans  = 0x0004 | 1,
    opXlat      = 0x0008,
    opAdd       = 0x0010,
    opFlipX     = 0x0020,
    opFlipY     = 0x0040,
};

class TBitBlock
{
public:
    TBitBlock( int width, int height ) : width_(width), height_(height) {
        data_ = new unsigned char [width*height];
    }

    /** Destructor. */
    virtual ~TBitBlock() {
        delete data_;
    }

    int width() const {
        return width_;
    }

    int height() const {
        return height_;
    }

    unsigned char pixel( int x, int y ) const {
        unsigned char result = 0;

        if( (x >= 0) && (x < width_) && (y >= 0) && (y < height_) ) {
            result = data_[x+y*width_];
        }

        return result;
    }

    void setPixel( int x, int y, unsigned char value ) {
        if( (x >= 0) && (x < width_) && (y >= 0) && (y < height_) ) {
            data_[x+y*width_] = value;
        }
    }

    void fill( unsigned char color ) {
        memset( data_, color, width_*height_ );
    }

    void clear() {
        fill( 0 );
    }

    TBitBlitter * createBlitter( unsigned op, unsigned char color, unsigned char trans, unsigned char * xlat );

    void copy( int x, int y, TBitBlock & block, int sx, int sy, int sw, int sh, unsigned op, TBitBlitter * blitter );

    void copy( int x, int y, TBitBlock & block, unsigned op, TBitBlitter * blitter ) {
        copy( x, y, block, 0, 0, block.width_, block.height_, op, blitter );
    }

    void copy( int x, int y, TBitBlock & block, unsigned op, unsigned char color, unsigned char trans, unsigned char * xlat ) {
        TBitBlitter * blitter = createBlitter( op, color, trans, xlat );
        copy( x, y, block, op, blitter );
        delete blitter;
    }

    unsigned char * data() const {
        return data_;
    }

private:
    int width_;
    int height_;
    unsigned char * data_;
};

#endif // EMU_BITBLOCK_H_
