/*
    Tickle class library

    Copyright (c) 2003,2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "emu_joystick2port.h"

enum {
    jpNone      = -1,
    jpLeft      = 0,
    jpRight     = 1,
    jpUp        = 2,
    jpDown      = 3,
    jpUpLeft    = 4,
    jpUpRight   = 5,
    jpDownLeft  = 6,
    jpDownRight = 7
};

TJoystickToPortHandler::TJoystickToPortHandler( unsigned device, int type, int ways )
{
    device_ = device;
    type_ = type;
    ways_ = ways;
    direction_ = jpNone;
    xpos_ = 0;
    ypos_ = 0;
    port_ = new unsigned char * [ways];
    and_mask_ = new unsigned char [ways];
    or_mask_ = new unsigned char [ways];
}

TJoystickToPortHandler::~TJoystickToPortHandler()
{
    delete port_;
    delete and_mask_;
    delete or_mask_;
}

void TJoystickToPortHandler::setPort( int index, unsigned char * port, unsigned char and_mask, unsigned char or_mask )
{
    if( (index >= 0) && (index < ways_) ) {
        port_[index] = port;
        and_mask_[index] = and_mask;
        or_mask_[index] = (or_mask != 0) ? or_mask : and_mask;
    }
}

void TJoystickToPortHandler::setDirection( int direction )
{
    if( direction_ != direction ) {
        for( int i=0; i<ways_; i++ ) {
            bool enabled = (i == direction);

            *(port_[i]) &= ~and_mask_[i];
            if( (enabled && (type_ == ptNormal)) || (!enabled && (type_ == ptInverted)) ) {
                *(port_[i]) |= or_mask_[i];
            }
        }

        direction_ = direction;
    }
}

bool TJoystickToPortHandler::handleInputEvent( unsigned device, unsigned param )
{
    bool result = false;

    if( device == device_ ) {
        int x = TInput::getXPosFromParam( param );
        int y = TInput::getYPosFromParam( param );

        if( (x != xpos_) || (y != ypos_) ) {
            result = onJoystickMoved( x, y );
            xpos_ = x;
            ypos_ = y;
        }
    }

    return result;
}

static int triChoice( int value, int on_negative, int on_zero, int on_positive )
{
    return (value > 0) ? on_positive : (value == 0) ? on_zero : on_negative;
}

TJoy4WayToPortHandler::TJoy4WayToPortHandler( unsigned device, int type )
    : TJoystickToPortHandler( device, type, 4 )
{
}

TJoy4WayToPortHandler::TJoy4WayToPortHandler( unsigned device, int type, unsigned char * port, unsigned masks )
    : TJoystickToPortHandler( device, type, 4 )
{
    setPortAndMasks( port, masks );
}

void TJoy4WayToPortHandler::setPortAndMasks( unsigned char * port, unsigned masks )
{
    for( int i=0; i<4; i++ ) {
        setPort( i, port, masks & 0xFF );
        masks >>= 8;
    }
}

bool TJoy4WayToPortHandler::onJoystickMoved( int x, int y )
{
    int direction;

    int xforce = TMath::abs( x );
    int yforce = TMath::abs( y );

    if( (xforce > yforce) || ((xforce == yforce) && (x != xpos())) ) {
        // Joystick moved on the X axis
        direction = triChoice( x, jpLeft, jpNone, jpRight );
    }
    else {
        // Joystick moved on the Y axis
        direction = triChoice( y, jpUp, jpNone, jpDown );
    }

    setDirection( direction );

    return true;
}
