/*
    Tickle class library

    Copyright (c) 2003,2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef EMU_RCFILTER_H_
#define EMU_RCFILTER_H_

class TRCFilter 
{
public:
    TRCFilter( double r, double c ) {
        setRC( r, c );
        y_ = 0.0;
    }

    void setRC( double r, double c ) {
        r_ = r;
        c_ = c;
        sampling_rate_ = 0; // Force a refresh next time filter is applied
    }

    void reset( double y = 0.0 ) {
        y_ = y;
    }

    void apply( int * buf, unsigned len, unsigned samplingRate ) {
        // Recalc coefficients if needed
        if( samplingRate != sampling_rate_ ) {
            sampling_rate_ = samplingRate;

            double d = r_ * c_ * sampling_rate_;
            
            b_ = exp( -1/d );
            a_ = 1 - b_;
        }

        while( len > 0 ) {
            y_ = a_ * (*buf) + b_ * y_;
            *buf++ = (int) y_;
            len--;
        }
    }

private:
    unsigned sampling_rate_;
    double y_; // Last sample
    double a_;
    double b_;
    double r_; // Resistor value in Ohms
    double c_; // Capacitor value in Farads
};

#endif // EMU_RCFILTER_H_
