#include "key_dispatcher.h"

struct KeyPressDispatcherItem
{
    unsigned key;
    unsigned id;
    unsigned data;
};

KeyPressDispatcher::KeyPressDispatcher()
{
    count_ = 0;
    capacity_ = 0;
    items_ = 0;
    joysticks_[0] = new TJoystick( idJoystick1 );
    joysticks_[1] = new TJoystick( idJoystick2 );
    joysticks_[2] = new TJoystick( idJoystick3 );
    joysticks_[3] = new TJoystick( idJoystick4 );
}

KeyPressDispatcher::~KeyPressDispatcher()
{
    delete [] items_;
    for( int i=0; i<4; i++ ) {
        delete joysticks_[i];
    }
}

void KeyPressDispatcher::expand()
{
    if( count_ == capacity_ ) {
        capacity_ += 16;
        KeyPressDispatcherItem * items = new KeyPressDispatcherItem[ capacity_ ];
        for( int i=0; i<count_; i++ ) {
            items[i] = items_[i];
        }
        delete [] items_;
        items_ = items;
    }
}

void KeyPressDispatcher::add( unsigned key, unsigned id, unsigned data )
{
    expand();

    // Find the position where this item must be inserted
    int index = 0;
    
    while( (index < count_) && (items_[index].key < key) ) {
        index++;
    }

    // Make place for the new item
    for( int i=count_; i>index; i-- ) {
        items_[i] = items_[i-1];
    }

    // Add the item
    items_[index].key = key;
    items_[index].id = id;
    items_[index].data = data;

    count_++;
}

void KeyPressDispatcher::setJoystick( int index, unsigned kleft, unsigned kright, unsigned kup, unsigned kdown )
{
    add( kleft, dcJoystick, (TJoystick::KeyLeft << 2) | index );
    add( kright, dcJoystick, (TJoystick::KeyRight << 2) | index );
    add( kup, dcJoystick, (TJoystick::KeyUp << 2) | index );
    add( kdown, dcJoystick, (TJoystick::KeyDown << 2) | index );
}

void KeyPressDispatcher::clear()
{
    count_ = 0;
}

bool KeyPressDispatcher::handle( unsigned key, unsigned param, TMachine * machine )
{
    bool result = false;

    int lo = 0;
    int hi = count_ - 1;

    // Do a binary search for the specified device
    while( lo <= hi ) {
        int i = (lo + hi) / 2;

        if( items_[i].key < key ) {
            lo = i + 1;
        }
        else {
            if( items_[i].key == key ) {
                // Found, dispatch the event
                unsigned id = items_[i].id;
                if( TInput::getDeviceClass( id ) == dcJoystick ) {
                    unsigned j = items_[i].data & 3;
                    unsigned k = items_[i].data >> 2;

                    joysticks_[j]->handleArrowKey( k, param );
                }
                else {
                    machine->handleInputEvent( id, items_[i].data | param, 0 );
                }
                result = true;
                break;
            }

            hi = i - 1;
        }
    }

    return result;
}
