/*
    Tickle class library

    Copyright (c) 2003,2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "emu_option2port.h"

struct TOptionItem {
    unsigned id;
    unsigned index;
    unsigned char * port;
    unsigned char and_mask;
    unsigned char or_mask;
};

TOptionToPortItem & TOptionToPortItem::operator << ( unsigned id )
{
    for( int i=0; i<ui_->groupCount(); i++ ) {
        TUiOptionGroup * group = ui_->group( i );

        for( int j=0; j<group->count(); j++ ) {
            TUiOption * option = group->item( j );
            unsigned option_class = TInput::getDeviceClass( option->id() );
            unsigned option_id = TInput::getDeviceId( option->id() );

            if( (option_class & dcOption) && (option_id == id) ) {
                for( int k=0; k<option->count(); k++ ) {
                    parent_->add( id, k, port_, option->mask(), option->itemValue(k) );
                }
            }
        }
    }

    return *this;
}

TOptionToPortHandler::~TOptionToPortHandler()
{
    clear();
}

void TOptionToPortHandler::clear()
{
    for( int i=0; i<options_.count(); i++ ) {
        TOptionItem * oi = (TOptionItem *) options_.item(i);

        delete oi;
    }

    options_.clear();
}

void TOptionToPortHandler::add( unsigned id, unsigned param, unsigned char * port, unsigned char and_mask, unsigned char or_mask )
{
    TOptionItem * opt = new TOptionItem();

    opt->id = id;
    opt->index = param;
    opt->port = port;
    opt->and_mask = and_mask;
    opt->or_mask = or_mask;

    options_.add( opt );
}

void TOptionToPortHandler::resynch( TUserInterface * ui )
{
    TList items;
    int i;

    // Move items into a temporary list
    for( i=0; i<options_.count(); i++ ) {
        items.add( options_.item(i) );
    }

    options_.clear();

    // Scan the list and re-add good items
    for( i=0; i<items.count(); i++ ) {
        TOptionItem * opt = (TOptionItem *) items.item(i);
        bool add = true;

        for( int j=0; j<options_.count(); j++ ) {
            TOptionItem * o = (TOptionItem *) options_.item(j);
            if( o->id == opt->id ) {
                add = false;
                break;
            }
        }

        if( add && ui->findOption(opt->id) ) {
            TOptionToPortItem opi( this, opt->port, ui );
            opi << opt->id;
        }

        delete opt;
    }
}
        
bool TOptionToPortHandler::handleInputEvent( unsigned device, unsigned param )
{
    bool result = false;

    if( device & dcOption ) {
        unsigned id = TInput::getDeviceId(device);

        for( int i=0; i<options_.count(); i++ ) {
            TOptionItem * opt = (TOptionItem *) options_.item(i);

            if( (opt->id == id) && (opt->index == param) ) {
                *(opt->port) &= ~opt->and_mask;
                *(opt->port) |= opt->or_mask;
                result = true;
                break;
            }
        }
    }

    return result;
}
