/*
    Tickle front-end
    User interface for Linux/Qt

    Copyright (c) 2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
void FormOptions::init()
{
    machine_ = 0;
}

void FormOptions::setMachine( TMachine * machine )
{
    machine_ = machine;
    options_.clear();
    listOptions->clear();
    listValues->clear();

    // The option list is arranged in groups, make it a flat list for our dialog box
    TUserInterface * ui = machine->getDriverInfo()->userInterface();
    int index;

    for( index=0; index<ui->groupCount(); index++ ) {
        TUiOptionGroup * group = ui->group(index);

        for( int i=0; i<group->count(); i++ ) {
            options_.add( group->item(i) );
            listOptions->insertItem( group->item(i)->name() );
        }
    }

    listOptions->setSelected( 0, true );
    listOptions->setCurrentItem( 0 );
}

void FormOptions::selectionChanged()
{
    int index = listOptions->index( listOptions->selectedItem() );

    TUiOption * option = (TUiOption *) options_.item( index );

    if( option != 0 ) {
        // Load values into list box
        listValues->clear();

        for( int i=0; i<option->count(); i++ ) {
            listValues->insertItem( option->item(i) );
        }

        listValues->setSelected( option->selected(), true );
        listValues->setCurrentItem( option->selected() );

        // Set caption
        QString caption;

        switch( TInput::getDeviceType(option->id()) ) {
        case dtDipSwitch:
            caption = "DIP switch:\r\n";
            break;
        case dtDriverOption:
            caption = "Driver option:\r\n";
            break;
        case dtCheat:
            caption = "Cheat:\r\n";
            break;
        case dtBoardSwitch:
            caption = "Board switch:\r\n";
            break;
        }

        caption += option->name();

        textLabelOption->setText( caption );
    }
}

void FormOptions::optionValueChanged()
{
    int index = listOptions->index( listOptions->selectedItem() );
    
    TUiOption * option = (TUiOption *) options_.item( index );
    
    int value = listValues->index( listValues->selectedItem() );
    
    if( (option != 0) && (value >= 0) ) {
        option->select( value );
        option->notify( machine_ );
    }
}

void FormOptions::restoreOptionDefault()
{
    int index = listOptions->index( listOptions->selectedItem() );
    
    TUiOption * option = (TUiOption *) options_.item( index );
    
    if( option != 0 ) {
        option->restoreDefault();
        listValues->setSelected( option->selected(), true );
    }
}

void FormOptions::closeEvent( QCloseEvent * event )
{
    emit formOptionsClosed();

    event->accept();
}
