/*
    Tickle front-end
    User interface for Linux/Qt

    Copyright (c) 2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <assert.h>
#include <string.h>

#include "linux_frame.h"

void LinuxFrame::setVideo( TBitmap * bitmap, bool flipped )
{
    if( bitmap != 0 ) {
        int width = TMath::min( bitmap->width(), screen_->width() );
        int height = TMath::min( bitmap->height(), screen_->height() );

        if( bitmap->format() == bfIndexed ) {
            TBitmapIndexed * bm = reinterpret_cast<TBitmapIndexed *>( bitmap );

            const unsigned * palette = bm->palette()->data();
            const unsigned char * bits = bm->bits()->data();

            if( flipped ) {
                bits += width*height;

                for( int y=0; y<height; y++ ) {
                    QRgb * pd = (QRgb *) screen_->scanLine( y );
                    for( int x=width; x>0; x-- ) {
                        *pd++ = palette[ *--bits ];
                    }
                }
            }
            else {
                for( int y=0; y<height; y++ ) {
                    QRgb * pd = (QRgb *) screen_->scanLine( y );
                    for( int x=width; x>0; x-- ) {
                        *pd++ = palette[ *bits++ ];
                    }
                }
            }
        }
        else if( bitmap->format() == bfRGB ) {
            TBitmapRGB * bm = reinterpret_cast<TBitmapRGB *>( bitmap );

            assert( sizeof(QRgb) == sizeof(unsigned) );

            const unsigned * bits = bm->data();

            for( int y=0; y<height; y++ ) {
                QRgb * pd = (QRgb *) screen_->scanLine( y );

                memcpy( pd, bits, sizeof(unsigned)*width );

                bits += width;
            }
        }
    }
}
