/*
    Tickle front-end
    User interface for Linux/Qt

    Copyright (c) 2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <assert.h>

#include <emu/emu.h>

#include "alsa_playback.h"
#include "linux_frame.h"

// For some reason g++ gets confused by the indirect
// reference and does not link the machines, so the whole point
// of not having to declare drivers here goes to... oh well...
#include <machine/frogger.h>
#include <machine/galaxian.h>
#include <machine/invaders.h>
#include <machine/pacman.h>
#include <machine/pooyan.h>
#include <machine/pinball_action.h>
#include <machine/scramble.h>

void dummy()
{
    delete Frogger::createInstance();
    delete Galaxian::createInstance();
    delete Pooyan::createInstance();
    delete Pacman::createInstance();
    delete PinballAction::createInstance();
    delete Scramble::createInstance();
    delete SpaceInvaders::createInstance();
}

const unsigned SamplesPerSecond = 44100;

// Global variables
char szAppName[] = "Tickle";    // Application name

static char szAbout[] =
        "Tickle\n"
        "Copyright (c) 1997-2003,2004 Alessandro Scotti\n"
        "\n"
        "http://www.ascotti.org/";

const char * PathSeparator = "/";

/* Tickle icon (PNG format) */
unsigned char icon_data[] = {
  0x89,0x50,0x4E,0x47,0x0D,0x0A,0x1A,0x0A,0x00,0x00,0x00,0x0D,0x49,0x48,0x44,0x52,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x10,0x04,0x03,0x00,0x00,0x00,0xED,0xDD,
  0xE2,0x52,0x00,0x00,0x00,0x0C,0x50,0x4C,0x54,0x45,0xCE,0x31,0x31,0xFF,0x9C,0x00,0xFF,0x9C,0x31,0xFF,0xFF,0xFF,0x6B,0xD2,0x34,0xBC,0x00,0x00,0x00,0x04,0x74,0x52,
  0x4E,0x53,0xFF,0xFF,0xFF,0x00,0x40,0x2A,0xA9,0xF4,0x00,0x00,0x00,0x34,0x49,0x44,0x41,0x54,0x78,0x9C,0x63,0x30,0x66,0x00,0x03,0x63,0x06,0x03,0x25,0x30,0x60,0x46,
  0x62,0x08,0x41,0x19,0x40,0x35,0x4A,0x4A,0x20,0x35,0xC6,0xC6,0x40,0x49,0x66,0x63,0x02,0x0C,0x26,0x25,0x25,0x05,0x02,0x0C,0xB0,0x1D,0xCC,0xC8,0x0C,0xB0,0x3B,0x8C,
  0x8D,0x01,0x80,0x41,0x10,0xE0,0x8D,0x49,0xC7,0x9D,0x00,0x00,0x00,0x00,0x49,0x45,0x4E,0x44,0xAE,0x42,0x60,0x82
};

TMachine * machine = 0;
bool active = false;
LinuxFrame * frame = 0;
TEmuInputManager inputManager;
FormMain * mainWindow = 0;
FormOptions * formOptions = 0;
int samplesPerFrame = 0;
AlsaPlayback * alsaPlayback = 0;
int sizeFactor = 0;
QTimer * timer = 0;

void setActive( bool a )
{
    if( active != a ) {
        active = a;

        QString caption( szAppName );

        if( ! active ) {
            caption += " - PAUSED";
        }

        mainWindow->setCaption( caption );

        if( alsaPlayback != 0 ) {
            if( active ) {
                alsaPlayback->start();
            }
            else {
                alsaPlayback->stop();
            }
        }
    }
}

// Load a file from an input stream into a newly allocated buffer
// (which must be deleted by the caller)
unsigned char * loadFileFromStream( TInputStream * is, unsigned size, TCRC32 * crc )
{
    unsigned char * result = new unsigned char [size];
    
    if( is->read( result, size ) == size ) {
        crc->reset();
        crc->update( result, size );
    }
    else {
        delete result;
        result = 0;
    }
    
    return result;
}

unsigned getFileSize( const char * name )
{
    unsigned result = 0;
    
    FILE * f = fopen( name, "rb" );

    if( f != 0 ) {
        fseek( f, 0L, SEEK_END );
        result = (unsigned) ftell( f );
        fclose( f );
    }
    
    return result;
}

// Load a single file required by the driver
unsigned char * loadFile( TMachine * machine, const TResourceFileInfo * info, const char * base, unsigned * bufsize )
{
    unsigned char * result = 0;
    
    TString home;

    if( base != 0 ) {
        home += base;
        home += PathSeparator;
    }
    
    TString file = home + info->name;
    TCRC32 crc;
    TInputStream * is = TFileInputStream::open( file.cstr() );
    
    if( is != 0 ) {
        unsigned size = info->size;
        
        if( size == 0 ) {
            size = getFileSize( file.cstr() );
        }
        
        result = loadFileFromStream( is, size, &crc );
        
        *bufsize = size;
        
        delete is;
    }
    else {
        for( int i=0; i<machine->getResourceCount(); i++ ) {
            file = home + machine->getResourceName(i) + ".zip";
            
            TZipFile * zf = TZipFile::open( file.cstr() );
            
            if( zf != 0 ) {
                const TZipEntry * ze = zf->entry( info->name, true );
                if( ze != 0 ) {
                    is = ze->open();

                    if( is != 0 ) {
                        *bufsize = info->size ? info->size : ze->size();
                        result = loadFileFromStream( is, *bufsize, &crc );
                    }

                    delete is;
                }

                delete zf;
            }

            if( result != 0 )
                break;
        }
    }
    
    if( result != 0 ) {
        // Delete buffer if CRC check fails
        if( (info->crc != 0) && (crc.value() != info->crc) ) {
            delete result;
            result = 0;
        }
    }

    return result;
}

// Load the files required by the driver
bool loadMachineFiles( TMachine * machine, TList & failedList )
{
    bool result = true;
    const TMachineDriverInfo * info = machine->getDriverInfo();
    
    for( int i=0; i<info->resourceFileCount(); i++ ) {
        const TResourceFileInfo * file = info->resourceFile( i );
        unsigned size = 0;
        unsigned char * buf = loadFile( machine, file, "roms", &size );
        
        if( (buf == 0) && (strstr(file->name,".wav") != 0) ) {
            buf = loadFile( machine, file, "samples", &size );
        }
        
        if( buf != 0 ) {
            machine->setResourceFile( file->id, buf, size );
            delete buf;
        }
        else {
            failedList.add( (void *)file->name );
            result = false;
        }
    }
    
    return result;
}

void resizeMainWindow()
{
    QImage * screen = frame->screen();
    
    int w = screen->width();
    int h = screen->height();
    
    if( sizeFactor == 1 ) {
        w *= 2;
        h *= 2;
    }
    
    mainWindow->resize( w, h + 3 + mainWindow->MenuBar->frameGeometry().height() );
}

bool loadGame( TMachine * m = 0 )
{
    bool result = false;
    
    setActive( false );
    
    // If we were not given a machine to load, let the user choose one
    if( m == 0 ) {
        FormLoadDriver * formLoad = new FormLoadDriver( mainWindow );
        
        if( formLoad->exec() ) {
            QListViewItem * selected = formLoad->listView->selectedItem();
            
            if( selected != 0 ) {
                int index = TGameRegistry::instance().find( selected->text(0) );
                
                const TGameRegistryItem * t = TGameRegistry::instance().item( index );
                
                if( t != 0 ) {
                    m = TMachine::createInstance( t->factory() );
                }
            }
        }
        
        delete formLoad;
    }
    
    // Load machine
    if( m != 0 ) {
        TList badFiles;
        bool ok = loadMachineFiles( m, badFiles );
        
        if( ! ok ) {
            // Unable to load required files
            QString msg = "One or more files could not be loaded:\n";
            
            for( int i=0; i<badFiles.count(); i++ ) {
                msg += "- ";
                msg += (char *) badFiles.item(i);
                msg += "\n";
            }
            
            msg += "\nFiles were looked for in the \"roms\" folder (also \"samples\""
                   "\nfor sounds) and in the following archives:\n";
            
            for( int j=0; j<m->getResourceCount(); j++ ) {
                msg += "- ";
                msg += m->getResourceName( j );
                msg += ".zip\n";
            }
            
            msg += "\nDo you want to start the game anyway?";
            
            if( QMessageBox::warning( mainWindow, szAppName, msg, QMessageBox::Yes, QMessageBox::No ) == QMessageBox::Yes ) {
                ok = true;
            }
            else {
                delete m;
            }
            
        }
        
        if( ok ) {
            // Replace current machine
            delete machine;
            machine = m;
            machine->reset();
            
            // Get info and update the GUI with the new parameters
            const TMachineDriverInfo * info = m->getDriverInfo();
            
            int w = info->machineInfo()->screenWidth;
            int h = info->machineInfo()->screenHeight;
            
            // Create a new frame buffer for this machine
            delete frame;
            frame = new LinuxFrame( w, h );
           
            // Resize the main window
            resizeMainWindow();
            
            // Create a new playback device
            delete alsaPlayback;
            samplesPerFrame = SamplesPerSecond / info->machineInfo()->framesPerSecond;
            alsaPlayback = new AlsaPlayback( SamplesPerSecond, samplesPerFrame );
            
            if( alsaPlayback->getLastError() < 0 ) {
                QMessageBox::critical( mainWindow, szAppName, "Cannot initialize ALSA driver!", QMessageBox::Ok, QMessageBox::NoButton );
            }
            
            // Update options dialog if still open
            if( formOptions != 0 ) {
                formOptions->setMachine( m );
            }

            result = true;
        }
    }

    setActive( true );

    return result;
}

void FormMain::fileLoad()
{
    loadGame();
}

void FormMain::fileExit()
{
    close();
}

void FormMain::gameRestart()
{
    machine->reset();
}

void FormMain::gameOptions()
{
    if( formOptions == 0 ) {
        formOptions = new FormOptions( this );

        connect( formOptions, SIGNAL( formOptionsClosed() ), this, SLOT( formOptionsClosed() ) );
    }

    formOptions->show();

    formOptions->setMachine( machine );
}

void FormMain::helpControls()
{
    char msg [] = "Left\tP1 Left\n"
                  "Right\tP1 Right\n"
                  "Up\tP1 Up\n"
                  "Down\tP1 Down\n"
                  "LCtrl\tP1 Action1\n"
                  "Z\tP1 Action2\n"
                  "X\tP1 Action3\n"
                  "C\tP1 Action4\n\n"
                  "J\tP2 Left\n"
                  "L\tP2 Right\n"
                  "I\tP2 Up\n"
                  "K\tP2 Down\n"
                  "E\tP2 Action1\n"
                  "R\tP2 Action2\n"
                  "T\tP2 Action3\n"
                  "G\tP2 Action4\n\n"
                  "1\tP1 Start\n"
                  "2\tP2 Start\n\n"
                  "5\tCoin slot 1\n"
                  "6\tCoin slot 2\n\n"
                  "P\tPause\n"
                  "F2\tToggle window size";
    
    QMessageBox::information( 0, "Controls", msg, QMessageBox::Ok );
}

void FormMain::helpAbout()
{
    QMessageBox::information( 0, szAppName, szAbout, QMessageBox::Ok );
}

void FormMain::init()
{
    mainWindow = this;

    statusBar()->hide();

    QPixmap * icon = new QPixmap();
    
    icon->loadFromData( icon_data, sizeof(icon_data) );

    setIcon( *icon );

    // Map controls
    inputManager.add( Qt::Key_Control, idKeyP1Action1 );
    inputManager.add( Qt::Key_Z, idKeyP1Action2 );
    inputManager.add( Qt::Key_X, idKeyP1Action3 );
    inputManager.add( Qt::Key_C, idKeyP1Action4 );

    inputManager.add( Qt::Key_E, idKeyP2Action1 );
    inputManager.add( Qt::Key_R, idKeyP2Action2 );
    inputManager.add( Qt::Key_T, idKeyP2Action3 );
    inputManager.add( Qt::Key_G, idKeyP2Action4 );

    inputManager.add( Qt::Key_1, idKeyStartPlayer1 );
    inputManager.add( Qt::Key_2, idKeyStartPlayer2 );
    inputManager.add( Qt::Key_5, idCoinSlot1 );
    inputManager.add( Qt::Key_6, idCoinSlot2 );

    inputManager.add( Qt::Key_0, idKeyService1 );

    // Player 1
    inputManager.addJoystick( idJoyP1Joystick1, Qt::Key_Left, Qt::Key_Right, Qt::Key_Up, Qt::Key_Down );
    inputManager.addJoystick( idJoyP1Joystick2, Qt::Key_A, Qt::Key_D, Qt::Key_W, Qt::Key_S );

    // Player 2
    inputManager.addJoystick( idJoyP2Joystick1, Qt::Key_J, Qt::Key_L, Qt::Key_I, Qt::Key_K );

    loadGame( TTickleMachine::create() );

    move( 16, 16 );

    timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );
    timer->start( 0 );
}

void doPaint( QPainter & painter )
{
    int y = 2 + mainWindow->MenuBar->frameGeometry().height();
    if( sizeFactor == 0 ) {
        painter.drawImage( 0, y, *frame->screen() );
    }
    else {
        QRect r( 0, y, 2*frame->screen()->width(), 2*frame->screen()->height() );
        painter.drawImage( r, *frame->screen() );
    }
}

void runOneFrame()
{
    if( ! active ) {
        return;
    }
    
    inputManager.notifyJoysticks( machine );
    
    // Draw current image
    QPainter painter( mainWindow );
    
    doPaint( painter );
    
    // Run the machine for a frame
    unsigned samplingRate = SamplesPerSecond;
    unsigned samplesPerFrame = samplingRate / machine->getDriverInfo()->machineInfo()->framesPerSecond;
    
    frame->getMixer()->clear();
    machine->run( frame, samplesPerFrame, samplingRate );
    
    // Write into the sound buffer
    if( alsaPlayback != 0 ) {
        int res = 0;
        
        res = alsaPlayback->synchAndPlay( frame->getMixer()->buffer(0), frame->maxVoices() );        

        if( res < 0 ) {
            setActive( false );
        }
    }
}

void FormMain::timerDone()
{
    runOneFrame();
}

void FormMain::keyPressEvent( QKeyEvent * k )
{
    if( inputManager.handle( k->key(), 1, machine ) ) {
        k->ignore();
    }
    else {
        switch( k->key() ) {
        case Qt::Key_Escape:
            close();
            break;
        case Qt::Key_P:
            setActive( ! active );
            break;
        case Qt::Key_F2:
            sizeFactor = 1-sizeFactor;
            resizeMainWindow();
            break;
        case Qt::Key_F4:
            QApplication::clipboard()->setImage( *frame->screen(), QClipboard::Clipboard );
            break;
        }
    }
}

void FormMain::keyReleaseEvent( QKeyEvent * k )
{
    if( inputManager.handle( k->key(), 0, machine ) ) {
        k->ignore();
    }
}

void FormMain::paintEvent( QPaintEvent * e )
{
    QPainter painter( mainWindow );
    
    painter.setClipRegion( e->region() );
    
    doPaint( painter );
}

void FormMain::formOptionsClosed()
{
    disconnect( formOptions, 0, 0, 0 );

    formOptions = 0;
}
